// MainDlg.h : interface of the CMainDlg class
//
/////////////////////////////////////////////////////////////////////////////

#pragma once

typedef std::map<long, CallInfo*> CallList;
typedef std::map<std::string, Tariff*> TariffMap;
typedef enum TariffType TariffType;

/**
* Main window class for the plugin.
*
* @author Rory Winston &lt;rory@researchkitchen.co.uk&gt;
*/
class CMainDlg : public CDialogImpl<CMainDlg>, public CUpdateUI<CMainDlg>,
	public CMessageFilter, public CIdleHandler,
	public IDispEventImpl<IDD_MAINDLG,CMainDlg,&DIID__ISkypeEvents,&LIBID_SKYPE4COMLib,1,0>
{
private:
	TariffCalculator* calculator;
	CallList calls;
	CStatic costLabel;
	float totalCost;
	IConversionPtr converter;
	static const int SKYPE_PROTOCOL_VERSION = 7;

public:
	enum { IDD = IDD_MAINDLG };
	ISkypePtr ptr;


	virtual BOOL PreTranslateMessage(MSG* pMsg)
	{
		return CWindow::IsDialogMessage(pMsg);
	}

	virtual BOOL OnIdle()
	{
		return FALSE;
	}

	BEGIN_SINK_MAP(CMainDlg)
		SINK_ENTRY_EX(IDD_MAINDLG, DIID__ISkypeEvents, 0x8, &OnCallStatusChange)
	END_SINK_MAP()

	BEGIN_UPDATE_UI_MAP(CMainDlg)
	END_UPDATE_UI_MAP()

	BEGIN_MSG_MAP(CMainDlg)
		MESSAGE_HANDLER(WM_INITDIALOG, OnInitDialog)
		COMMAND_ID_HANDLER(ID_APP_ABOUT, OnAppAbout)
		COMMAND_ID_HANDLER(IDOK, OnOK)
		COMMAND_ID_HANDLER(IDCANCEL, OnCancel)
	END_MSG_MAP()


	// Handler prototypes (uncomment arguments if needed):
	//	LRESULT MessageHandler(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
	//	LRESULT CommandHandler(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	//	LRESULT NotifyHandler(int /*idCtrl*/, LPNMHDR /*pnmh*/, BOOL& /*bHandled*/)

	LRESULT OnInitDialog(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
	{
		// center the dialog on the screen
		CenterWindow();

		// set icons
		HICON hIcon = (HICON)::LoadImage(_Module.GetResourceInstance(), MAKEINTRESOURCE(IDR_MAINFRAME), 
			IMAGE_ICON, ::GetSystemMetrics(SM_CXICON), ::GetSystemMetrics(SM_CYICON), LR_DEFAULTCOLOR);
		
		SetIcon(hIcon, TRUE);
		
		HICON hIconSmall = (HICON)::LoadImage(_Module.GetResourceInstance(), MAKEINTRESOURCE(IDR_MAINFRAME), 
			IMAGE_ICON, ::GetSystemMetrics(SM_CXSMICON), ::GetSystemMetrics(SM_CYSMICON), LR_DEFAULTCOLOR);
		SetIcon(hIconSmall, FALSE);

		costLabel = GetDlgItem(IDC_STC3);
		totalCost = 0.0f;

		// register object for message filtering and idle updates
		CMessageLoop* pLoop = _Module.GetMessageLoop();
		ATLASSERT(pLoop != NULL);
		pLoop->AddMessageFilter(this);
		pLoop->AddIdleHandler(this);

		UIAddChildWindowContainer(m_hWnd);

		/// Load and parse the tariffs file
		/// The tariffs file is a simple pipe-delimited text file that has been 
		/// downloaded from the BT website (see http://http://www.researchkitchen.co.uk/blog/archives/70 for details)
		calculator = new TariffCalculator();
		std::string tariffsFile("tariffs.dat");
		calculator->parse(tariffsFile);

		// Initialize an instance of the Skype COM 
		HRESULT hr;
		hr = ptr.CreateInstance(__uuidof(Skype));
		if (FAILED(hr)) {
			MessageBox("Failed to create Skype instance!\n", "Error");
			exit(-1);
		}
		
		/// Create an instance of the IConversion interface. 
		/// This translates internal Skype codes into human-readable form
		converter.CreateInstance(__uuidof(Conversion));
		
		try {
			OutputDebugString("Connecting...\n");
			//hr = ptr->Client->Start(false, false);
			hr = ptr->Attach(SKYPE_PROTOCOL_VERSION, true);
			if (SUCCEEDED(hr)) {
				OutputDebugString("Connected to Skype\n");
			}

			// Hook up for event notifications
			hr = DispEventAdvise(ptr);
			if (FAILED(hr)) {
				MessageBox("Failed to connect event handlers!\n", "Error");
				exit(-1);
			}
		}
		catch (_com_error e) {
			_tprintf(_T("There was an error: %s %s %s\n"),
				e.Error(), e.ErrorMessage(), e.Description());
		}

		return TRUE;
	}

	LRESULT OnAppAbout(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		CAboutDlg dlg;
		dlg.DoModal();
		return 0;
	}

	LRESULT OnOK(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		CloseDialog(wID);
		return 0;
	}

	LRESULT OnCancel(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		CloseDialog(wID);
		return 0;
	}

	void CloseDialog(int nVal)
	{
		/// Unhook the event handlers
		DispEventUnadvise(ptr);

		/// 
		TariffMap tariffs = calculator->getTariffs();
		for (TariffMap::iterator it = tariffs.begin();
			it != tariffs.end(); it++) {
				Tariff* tariff = it->second;
				delete tariff;
		}

		delete calculator;

		/// Free any allocated CallInfo* structures (and the associated string pointers)
		for (CallList::iterator it = calls.begin();
			it != calls.end(); it++) {
				CallInfo* callInfo = it->second;
				delete callInfo->callerCountry;
				delete callInfo;
		}


		DestroyWindow();
		::PostQuitMessage(nVal);
	}

	// Event handlers
	void __stdcall OnCallStatusChange(ICall* pCall, enum TCallStatus status) {
		std::ostringstream str;

		USES_CONVERSION;
		
		/// A call has been initiated.
		if (status == clsInProgress) {
			str << "Call in progress with " << (LPCTSTR)OLE2T(pCall->PartnerHandle.GetBSTR());
			IUserPtr user = ptr->GetUser(pCall->PartnerHandle);

			CallInfo* callInfo = new CallInfo();
			callInfo->callId = pCall->Id;
			callInfo->duration = 0L;

			std::string* country = new std::string(OLE2T(user->Country.GetBSTR()));
			callInfo->callerCountry = country;

			TariffType tariffType = getTariffType();
			callInfo->tariffType = tariffType;

			calls[pCall->Id] = callInfo;
		}
		else if (status == clsRouting) {
			str << "Routing ...";
		}
		else if (status == clsRinging) {
			str << "Ringing ...";
		}
		else if (status == clsFailed) {
			str << "Call failed, reason code = " << pCall->FailureReason;
		}
		else if (status == clsFinished) {
			str << "Call # " << pCall->Id << " finished, duration: " << pCall->Duration << " seconds.";
			
			// Retrieve the call from the active calls list
			CallList::const_iterator it = calls.find(pCall->Id);
			if (it == calls.end()) {
				OutputDebugString("Call Id not found!");
				return;
			}

			CallInfo* callInfo = it->second;
			callInfo->duration = pCall->Duration;
			float cost = calculator->getCost(callInfo);
			totalCost += cost;
			std::ostringstream ostr;
			ostr << totalCost;
	
			costLabel.SetWindowText(ostr.str().c_str());
			
			// Remove the call from the active calls map
			calls.erase(pCall->Id);
		}

		CListBox listbox = GetDlgItem(IDC_LST1);
		listbox.AddString(str.str().c_str());
		OutputDebugString(str.str().c_str());
	}

	/**
	* Deduce the appropriate tariff type from the current date and time
	*/
	TariffType getTariffType(void) {
		using namespace boost::posix_time;
		using namespace boost::gregorian;
		ptime now = second_clock::local_time();

		date d = now.date();
		greg_weekday day = d.day_of_week();
		
		unsigned short weekday = day.as_number();
		if (weekday == 0 || weekday == 6) {
			return TariffType::WEEKEND;
		}

		time_duration t = now.time_of_day();
		long hours = t.hours();
		if (hours >= 6 && hours <= 18) {
			return TariffType::DAY;
		}
		else {
			return TariffType::EVENING;
		}
	}

	virtual HRESULT GetFuncInfoFromId(const IID& iid, DISPID dispidMember, LCID lcid, _ATL_FUNC_INFO& info) {
		// class base class implementation
		HRESULT hr = IDispEventImpl<IDD_MAINDLG, CMainDlg,
			&DIID__ISkypeEvents, &LIBID_SKYPE4COMLib, 1, 0>::
			GetFuncInfoFromId(iid, dispidMember,lcid, info);
		if (SUCCEEDED(hr)) {
			if (::InlineIsEqualGUID(iid, DIID__ISkypeEvents)) {
				for(long l = 0; l < info.nParams; l++) {
					if(info.pVarTypes[l] == VT_USERDEFINED) {
						info.pVarTypes[l] = VT_I4;
					}
				}
			}
		}
		return hr;
	}
};
